
#include "HX711.h"

// Define the number of load cells
const int NUM_CELLS = 2;

// Define the data pins for each load cell
const int LOADCELL_DOUT_PINS[NUM_CELLS] = {D4, D0};

// Define the shared clock pin for all load cells
const int LOADCELL_SCK_PIN = D3;

HX711 loadCells[NUM_CELLS];  // Array to store HX711 instances
float offsets[NUM_CELLS];    // Offset values for calibration

void setup() {
  Serial.begin(115200);
  
  Serial.println("Initializing load cells...");
  for (int i = 0; i < NUM_CELLS; i++) {
    loadCells[i].begin(LOADCELL_DOUT_PINS[i], LOADCELL_SCK_PIN);
    delay(1000);  // Allow stabilization time for each load cell
  }

  Serial.println("Taring and calibrating all load cells...");
  calibrateAll();  // Perform initial tare and calibration

  Serial.println("System ready. Send 'T' to recalibrate.");
}

void loop() {
  // Check for user input to manually recalibrate
  if (Serial.available()) {
    char command = Serial.read();
    if (command == 'T' || command == 't') {
      Serial.println("Re-taring and recalibrating all load cells...");
      calibrateAll();
    }
  }

  // Start reading process
  Serial.print("Readings: ");

  for (int i = 0; i < NUM_CELLS; i++) {
    if (loadCells[i].is_ready()) {
      delay(200);  // Ensure stability before reading
      float rawReading = loadCells[i].get_units();  // Scaled reading
      float calibratedReading = rawReading - offsets[i];  // Apply offset correction

      // Ensure readings are positive
      calibratedReading = abs(calibratedReading);

      Serial.print("LoadCell_");
      Serial.print(i + 1);
      Serial.print(":");
      Serial.print(calibratedReading);

    } else {
      Serial.print("LoadCell_");
      Serial.print(i + 1);
      Serial.print(":0");  // Prevent plotter errors by sending a default value
    }

    if (i < NUM_CELLS - 1) {
      Serial.print("\t");  // Tab separation for Serial Plotter
    }

    delay(200);  // Additional delay for stability between readings
  }

  Serial.println();  // Newline for Serial Plotter formatting
  delay(500);  // Delay before the next reading cycle
}

// Function to tare and calibrate all load cells
void calibrateAll() {
  for (int i = 0; i < NUM_CELLS; i++) {
    Serial.print("Taring load cell ");
    Serial.println(i + 1);
    loadCells[i].tare();  // Perform automatic taring
    delay(500);  // Ensure stability
    offsets[i] = loadCells[i].get_units();  // Store correct offset value
  }
  Serial.println("Taring and calibration completed.");
}